// File:		reporter.CPP

//============================================================================
// Content:			Implementation of class HReporter
// Author:			James Huang
// Date:			Oct. 1994
//============================================================================

#include "reporter.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

short HReporter::CRPE_Count = 0;

LPSTR Title(LPSTR title) {return title?title:"Report Preview";}

//----------------------------------------------------------------------------
// Constructors, destructors, protected and private functions
//----------------------------------------------------------------------------

//   Sets default values for the internal data members and
//   opens the print engine, if necessary, and register this object.
HReporter::HReporter (LPSTR reportPath) {
	text = NULL;
	printJob = 0;
	EnableErrorHandling ();
	RegisterToEngine ();
	if (reportPath) OpenReport (reportPath);
}

HReporter::~HReporter () {
	CloseReport ();
	if (IsEngineOpened() && 0==(--CRPE_Count))
		PECloseEngine ();
	if (text) free(text);
}

BOOL HReporter::GetHandleString () {
	if(text) free(text);
	if (len>0) {
		text = (char*)malloc(len);
		text[0] = 0;
		return PEGetHandleString(handle,text,len);
	} else return FALSE;
}

void HReporter::RegisterToEngine () {
	BOOL flag = TRUE;
	if (!IsEngineOpened()) {
		CRPE_Count = 0;
		flag = PEOpenEngine ();
	}
	if (flag) CRPE_Count++;
	else Error ("ERROR OPEN ENGINE!");
}

void  HReporter::Error (LPCSTR errString) {
	char buf[500];
	if (doError) {
		LPCSTR ptr = errString;
		if (printJob && PEGetErrorCode(printJob)) {	// if no error, by-pass.
			if (IsEngineOpened()) {
				// get engine error text for the last engine operation.
				PEGetErrorText (printJob, &handle, &len);
				GetHandleString (); // to text
			} else {
				if (text) free(text);
				text = _fstrdup("Print Engine Not Open");
			}
			sprintf (buf, "%s !\n\nCRPE Message:\n   %s", errString, text);
			ptr = buf;
		}
		::MessageBox (::GetActiveWindow(),
					  ptr,"Reporter Message",MB_OK|MB_ICONSTOP);
	}
}

//----------------------------------------------------------------------------
//	Public functions
//----------------------------------------------------------------------------

BOOL HReporter::OpenReport (LPSTR reportPath) {
	HCURSOR hCursor = SetCursor (LoadCursor (0, IDC_WAIT));
	BOOL flag = IsEngineOpened ();
	if (flag) {
		if (printJob) CloseReport ();
		flag = (printJob=PEOpenPrintJob(reportPath));
	}
	SetCursor (hCursor);
	char buf[200];
	sprintf (buf, "ERROR OPEN PRINT JOB FOR \"%s\"", reportPath);
	if (!flag) Error(buf);
	return flag;
}

BOOL HReporter::CanCloseReport () {
	struct PEJobInfo jI;
	return !((printJob||IsWindowOn()) && GetJobStatus(&jI)==PE_JOBINPROGRESS);
}

BOOL HReporter::CloseReport () {
	if (CloseWindow()) {
		PEClosePrintJob (printJob);
		printJob = 0;
		return TRUE;
	} else return FALSE;
}

BOOL HReporter::CloseWindow () {
	BOOL flag;
	if (flag = !CanCloseReport())
		Error ("Last Job Still In Print");
	else {
		if (wndAttr.HWindow()) SendMessage(wndAttr.HWindow(),WM_CLOSE,0,0L);
		wndAttr.Clear();
	}
	return flag;
}

BOOL HReporter::DoPrint (int nCopies) {
	if (!printJob) return FALSE;
	HCURSOR hCursor = SetCursor (LoadCursor (0, IDC_WAIT));
	BOOL flag = (IsWindowOn() && PEGetWindowHandle (printJob)) ?
		!PEPrintWindow (printJob,1) :
		!(PEOutputToPrinter(printJob,nCopies) && PEStartPrintJob(printJob,1));
	SetCursor (hCursor);
	if (flag) Error ("ERROR PRINT REPORT" );
	return flag;
}

BOOL HReporter::OutputToWindow (LPSTR title,HWND parent,LONG style,
		  short left, short top, short width, short height) {
	if (!printJob) return FALSE;
	HCURSOR hCursor = SetCursor (LoadCursor (0, IDC_WAIT));
	BOOL flag = PEOutputToWindow(printJob,Title(title),left,top,width,
						height,style,parent) && PEStartPrintJob(printJob,1);
	wndAttr.hWindow = flag ? PEGetWindowHandle(printJob) : 0;
	SetCursor (hCursor);
	return flag;
}

BOOL HReporter::DoMDIPreview (HWND hClient,LPSTR title) {
	HWND hMDIChild;
	RECT rc;
	if (!printJob || IsWindowOn()) return FALSE;
	wndAttr.hParent = hMDIChild = wndAttr.CreateMDIChild(hClient,title);
	if (!hMDIChild) return 0;
	GetClientRect(hMDIChild,&rc);
	BOOL flag = OutputToWindow (Title(title),hMDIChild,WS_CHILD,
								rc.left,rc.top,rc.right,rc.bottom);
	if (flag) return TRUE;
	SendMessage(hMDIChild,WM_CLOSE,0,0L);
	wndAttr.Clear();
	return 0;
}

//----------------------------------------------------------------------------
// HReporter::WindowAttr::CreateMDIChild()
//----------------------------------------------------------------------------

LPSTR HCRPEChildClassName = "CRPEChild";

#include "owlchild.cpp"
//#include "mfcchild.cpp"
//#include "justchld.cpp"

